# Copyrights 2001-2007 by Mark Overmeer.
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 1.02.

use strict;
use warnings;

package Mail::Server;
use vars '$VERSION';
$VERSION = '2.073';
use base 'Mail::Reporter';


1;
