#!/usr/bin/perl -T
#
# Test threads over multi-folders.
#

use strict;
use warnings;

use lib qw(. .. tests);
use Tools;

use Test::More tests => 1;

#use Mail::Box::Manager;

pass("   * Multi-folder threads status ALPHA (not tested)");
