# Copyrights 2001-2007 by Mark Overmeer.
#  For other contributors see ChangeLog.
# See the manual pages for details on the licensing terms.
# Pod stripped from pm file by OODoc 1.02.

package MailBox::Test::53threads::Definition;
use vars '$VERSION';
$VERSION = '2.073';

sub name     {"Mail::Box::Threads; message threads"}
sub critical {0}
sub skip     { undef }

1;
